function L = assemLoad_1D(Coordinates,QuadRule,FHandle,varargin)
% ASSEMLOAD_1D Assemble linear FE contributions.
%
%   L = ASSEMLOAD_1D(COORDINATES,QUADRULE,FHANDLE) assembles the global
%   load vector for the load data given by the function handle FHANDLE.
%
%   COORDINATES  is a N-by-2 matrix specifying the vertices of the mesh.
%
%   QUADRULE is a struct, which specifies the Gauss qaudrature that is
%   used to do the integration:
%    w Weights of the Gauss quadrature.
%    x Abscissae of the Gauss quadrature.  
%
%   L = ASSEMLOAD_1D(COORDINATES,QUADRULE,FHANDLE,FPARAM) also handles
%   the additional variable length argument list FPARAM to the function
%   handle FHANDLE.
%
%   Example:
%
%   Coordinates = transpose(0:.01:1);
%   FHandle = @(x,varargin)8*ones(size(x));
%   L = assemLoad_1D(Coordinates,gauleg(0,1,5),FHandle);
%
%   See also shap_1D.

%   Copyright 2005-2005 Patrick Meury
%   SAM - Seminar for Applied Mathematics
%   ETH-Zentrum
%   CH-8092 Zurich, Switzerland

  % Initialize constants
  
  nCoordinates = size(Coordinates,1);
  
  % Preallocate memory
  
  L = zeros(nCoordinates,1);
  Lloc = zeros(2,1);
  
  % Precompute shape functions
  
  N = shap_1D(QuadRule.x);
  
  % Assemble element contributions
  vidx = [1 2];
  for i = 1:(nCoordinates-1)

     % Compute element mapping
     
     bK = Coordinates(vidx(1));
     BK = Coordinates(vidx(2))-bK;
     det_BK = abs(BK);
     
     x = QuadRule.x*BK + bK;
  
     % Compute load data
  
     FVal = FHandle(x,varargin{:});
  
     % Compute element load vector
  
     Lloc(1) = sum(QuadRule.w.*FVal.*N(:,1))*det_BK;
     Lloc(2) = sum(QuadRule.w.*FVal.*N(:,2))*det_BK; 
    
     % Add contributions to global load vector
    
     L(vidx) = L(vidx) + Lloc;
        
     % Update current element 
    
     vidx = vidx+1;
    
  end

return