% Run script for 1D finite element solver on graded meshes.

%   Copyright 2008 Christoph Winter
%   SAM - Seminar for Applied Mathematics
%   ETH-Zentrum

  close all
  
  % Initialize constants
  NREFS = 10;                                         % Number of refinement steps
  XL = 0;                                             % Left-hand side boundary
  XR = 1;                                             % Right-hand side boundary
  F_HANDLE = @(x,alpha)alpha*(1-alpha)*x.^(alpha-2);  % Right-hand side source term
  GD_HANDLE = @(x,alpha)x.^alpha;                     % Dirichlet boundary data
  U_EX = @(x,alpha)x.^alpha;                          % Exact solution   
  DU_EX = @(x,alpha)alpha.*x.^(alpha-1);              % Exact derivative of solution 
  ALPHA = [1.1 1.3 1.5];                              % Solution parameter
  BETA = 2./(2*ALPHA-1);                              % Grading factor
        
  % Loop over different values of alpha
  N = zeros(NREFS,1);
  H1_Error = zeros(NREFS,length(ALPHA));
  for j=1:length(ALPHA)
      
      MESH_WIDTH = 1e-1;                              % Mesh resolution  
      
      % Compute discretization error on a series of meshes
      for i = 1:NREFS
      
          % Compute mesh
          MESH_WIDTH = MESH_WIDTH/2;
          Coordinates = transpose(XL:MESH_WIDTH:XR).^BETA(j);
  
          % Compute stiffness matrix and load vector
          A = assemMat_1D(Coordinates,@STIMA_Lapl_1D);
          L = assemLoad_1D(Coordinates,gauleg(0,1,10),F_HANDLE,ALPHA(j));
    
          % Incorporate Dirichlet boundary data
          [U,FreeDofs] = assemDir_1D(Coordinates,[1 size(Coordinates,1)],GD_HANDLE,ALPHA(j));
          L = L - A*U;
    
          % Solve the linear system
          U(FreeDofs) = A(FreeDofs,FreeDofs)\L(FreeDofs);

          % Compute discretization error
          H1_Error(i,j) = H1Err_1D(Coordinates,U,gauleg(0,1,10),U_EX,DU_EX,ALPHA(j));
          N(i) = length(Coordinates);
      end
  end

% plot rates
figure(1);
h = axes;
color  = ['bo-';'g+-';'rx-';'m*-';'cs-';'yd-';'kp-'];
lstr = cell(length(ALPHA),1);
for i=1:length(ALPHA)
    loglog(N,H1_Error(:,i),color(i,:))
    polyfit(log(N),log(H1_Error(:,i)),1)
    lstr{i} = ['alpha = ' num2str(ALPHA(i)) ', beta = ' num2str(BETA(i))];
    hold on
end
grid('on');
set(h,'FontSize',14);
xlabel('Dofs [log]');
ylabel('Discretization error [log]');
legend(lstr,'Location','NorthEast')

add_Slope(gca,'SouthWest',-1);
  
clear all  
